==> rspamd_3.12.1.orig.tar.xz: rspamd-3.12.1/src/libserver/http/http_context.h <==
/*-
 * Copyright 2019 Vsevolod Stakhov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef RSPAMD_HTTP_CONTEXT_H
#define RSPAMD_HTTP_CONTEXT_H

#include "config.h"
#include "ucl.h"
#include "addr.h"

#include "contrib/libev/ev.h"

#ifdef __cplusplus
extern "C" {
#endif

struct rspamd_http_context;
struct rspamd_config;
struct rspamd_http_message;
struct upstream_ctx;

struct rspamd_http_context_cfg {
	unsigned int kp_cache_size_client;
	unsigned int kp_cache_size_server;
	unsigned int ssl_cache_size;
	double keepalive_interval;
	double client_key_rotate_time;
	const char *user_agent;
	const char *http_proxy;
	const char *server_hdr;
};

/**
 * Creates and configures new HTTP context
 * @param root_conf configuration object
 * @param ev_base event base
 * @return new context used for both client and server HTTP connections
 */
struct rspamd_http_context *rspamd_http_context_create(struct rspamd_config *cfg,
													   struct ev_loop *ev_base,
													   struct upstream_ctx *ctx);

struct rspamd_http_context *rspamd_http_context_create_config(
	struct rspamd_http_context_cfg *cfg,
	struct ev_loop *ev_base,
	struct upstream_ctx *ctx);

/**
 * Destroys context
 * @param ctx
 */
void rspamd_http_context_free(struct rspamd_http_context *ctx);

struct rspamd_http_context *rspamd_http_context_default(void);

/**
 * Returns preserved keepalive connection if it's available.
 * Refcount is transferred to caller!
 * @param ctx
 * @param addr
 * @param host
 * @return
 */
struct rspamd_http_connection *rspamd_http_context_check_keepalive(struct rspamd_http_context *ctx,
																   const rspamd_inet_addr_t *addr,
																   const char *host,
																   bool is_ssl);

/**
 * Checks if there is a valid keepalive connection
 * @param ctx
 * @param addr
 * @param host
 * @param is_ssl
 * @return
 */
const rspamd_inet_addr_t *rspamd_http_context_has_keepalive(struct rspamd_http_context *ctx,
															const char *host,
															unsigned port,
															bool is_ssl);

/**
 * Prepares keepalive key for a connection by creating a new entry or by reusing existent
 * Bear in mind, that keepalive pool has currently no cleanup methods!
 * @param ctx
 * @param conn
 * @param addr
 * @param host
 */
void rspamd_http_context_prepare_keepalive(struct rspamd_http_context *ctx, struct rspamd_http_connection *conn,
										   const rspamd_inet_addr_t *addr, const char *host, bool is_ssl);

/**
 * Pushes a connection to keepalive pool after client request is finished,
 * keepalive key *must* be prepared before using of this function
 * @param ctx
 * @param conn
 * @param msg
 */
void rspamd_http_context_push_keepalive(struct rspamd_http_context *ctx,
										struct rspamd_http_connection *conn,
										struct rspamd_http_message *msg,
										struct ev_loop *ev_base);

#ifdef __cplusplus
}
#endif

#endif

==> rspamd_3.13.2.orig.tar.xz: rspamd-3.13.2/src/libserver/http/http_context.h <==
/*-
 * Copyright 2019 Vsevolod Stakhov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef RSPAMD_HTTP_CONTEXT_H
#define RSPAMD_HTTP_CONTEXT_H

#include "config.h"
#include "ucl.h"
#include "addr.h"

#include "contrib/libev/ev.h"

#ifdef __cplusplus
extern "C" {
#endif

struct rspamd_http_context;
struct rspamd_config;
struct rspamd_http_message;
struct upstream_ctx;

struct rspamd_http_context_cfg {
	unsigned int kp_cache_size_client;
	unsigned int kp_cache_size_server;
	unsigned int ssl_cache_size;
	double keepalive_interval;
	double client_key_rotate_time;
	const char *user_agent;
	const char *http_proxy;
	const char *server_hdr;
	/* Client-side staged timeouts (seconds) */
	double connect_timeout; /* TCP connect */
	double ssl_timeout;     /* SSL handshake */
	double write_timeout;   /* Request write */
	double read_timeout;    /* Response read */
	/* Keep-alive/pool tuning */
	unsigned int keepalive_pool_size; /* max conns per key */
	double keepalive_connection_ttl;  /* absolute TTL */
	double keepalive_idle_timeout;    /* idle timeout */
	unsigned int keepalive_max_reuse; /* reuse limit */
	int keepalive_eviction_policy;    /* 0=LIFO,1=LRU */
};

/**
 * Creates and configures new HTTP context
 * @param root_conf configuration object
 * @param ev_base event base
 * @return new context used for both client and server HTTP connections
 */
struct rspamd_http_context *rspamd_http_context_create(struct rspamd_config *cfg,
													   struct ev_loop *ev_base,
													   struct upstream_ctx *ctx);

struct rspamd_http_context *rspamd_http_context_create_config(
	struct rspamd_http_context_cfg *cfg,
	struct ev_loop *ev_base,
	struct upstream_ctx *ctx);

/**
 * Destroys context
 * @param ctx
 */
void rspamd_http_context_free(struct rspamd_http_context *ctx);

struct rspamd_http_context *rspamd_http_context_default(void);

/**
 * Returns preserved keepalive connection if it's available.
 * Refcount is transferred to caller!
 * @param ctx
 * @param addr
 * @param host
 * @return
 */
struct rspamd_http_connection *rspamd_http_context_check_keepalive(struct rspamd_http_context *ctx,
																   const rspamd_inet_addr_t *addr,
																   const char *host,
																   bool is_ssl);

/**
 * Checks if there is a valid keepalive connection
 * @param ctx
 * @param addr
 * @param host
 * @param is_ssl
 * @return
 */
const rspamd_inet_addr_t *rspamd_http_context_has_keepalive(struct rspamd_http_context *ctx,
															const char *host,
															unsigned port,
															bool is_ssl);

/**
 * Prepares keepalive key for a connection by creating a new entry or by reusing existent
 * Bear in mind, that keepalive pool has currently no cleanup methods!
 * @param ctx
 * @param conn
 * @param addr
 * @param host
 */
void rspamd_http_context_prepare_keepalive(struct rspamd_http_context *ctx, struct rspamd_http_connection *conn,
										   const rspamd_inet_addr_t *addr, const char *host, bool is_ssl);

/**
 * Pushes a connection to keepalive pool after client request is finished,
 * keepalive key *must* be prepared before using of this function
 * @param ctx
 * @param conn
 * @param msg
 */
void rspamd_http_context_push_keepalive(struct rspamd_http_context *ctx,
										struct rspamd_http_connection *conn,
										struct rspamd_http_message *msg,
										struct ev_loop *ev_base);

#ifdef __cplusplus
}
#endif

#endif
